@@ -12,7 +12,6 @@
 #' @inheritParams server_config
 #' @param ... server configurations passed to \code{\link{server_config}()}
 #' @export
-#' @importFrom httpuv runServer startDaemonizedServer
 #' @references \url{https://github.com/yihui/servr}
 #' @examples #' see https://github.com/yihui/servr for command line usage
 #' # or run inside an R session
@@ -94,7 +93,8 @@ watch_dir = function(dir = '.', pattern = NULL, all_files = FALSE, handler = NUL
 #'   global option \code{getOption('servr.daemon')} (e.g., you can set
 #'   \code{options(servr.daemon = TRUE)}); if this option was not set,
 #'   \code{daemon = TRUE} if a command line argument \code{-d} was passed to R
-#'   (through \command{Rscript}); normally it should be \code{FALSE} by default
+#'   (through \command{Rscript}), or the server is running in an interactive R
+#'   session
 #' @param interval the time interval used to check if an HTML page needs to be
 #'   rebuilt (by default, it is checked every second); at the moment, the
 #'   smallest possible \code{interval} is set to be 1, and this may change in
@@ -116,48 +116,27 @@ server_config = function(
   if (missing(port))
     port = if (length(port <- grep('^-p[0-9]{4,}$', cargs, value = TRUE)) == 1)
       as.integer(sub('^-p', '', port)) else random_port()
-  if (missing(daemon)) daemon = getOption('servr.daemon', '-d' %in% cargs)
+  if (missing(daemon)) daemon = getOption('servr.daemon', ('-d' %in% cargs) || interactive())
   damn_library('methods')
   url = sprintf('http://%s:%d', host, port)
   if (baseurl != '') url = paste(url, baseurl, sep = '')
   url = paste0(url, if (initpath != '' && !grepl('^/', initpath)) '/', initpath)
   browsed = FALSE
-  browse = function() {
-    if (browsed) return(invisible(url))
-    if (browser) {
-      browseURL(url, browser = get_browser())
-    }
+  servrEnv$browse = browse = function(reopen = FALSE) {
+    if (browsed && !reopen) return(invisible(url))
+    if (browser || reopen) browseURL(url, browser = get_browser())
     browsed <<- TRUE
-    message('Serving the directory ', dir, ' at ', url)
+    if (!reopen) message('Serving the directory ', dir, ' at ', url)
   }
   list(
     host = host, port = port, interval = interval, url = url,
     start_server = function(app) {
-      # a built-in daemonized server
-      if (daemon && !requireNamespace('later', quietly = TRUE)) {
-        id = startDaemonizedServer(host, port, app)
-        browse()
-        return(daemon_hint(id))
-      }
-
-      server = startServer(host, port, app)
-      browse()
-      # a daemonized server based on later
-      if (daemon) {
-        daemon_hint(server, TRUE)
-        daemon_serve = function() {
-          if (!server %in% daemon_list()) return()
-          service()
-          later::later(daemon_serve, 0.01)
-        }
-        daemon_serve()
-      } else {
-        on.exit(stopServer(server), add = TRUE)
-        while (TRUE) {
-          service()
-          Sys.sleep(0.001)
-        }
+      id = startServer(host, port, app)
+      if (daemon) daemon_hint(id); browse()
+      if (!daemon) while (TRUE) {
+        httpuv::service(); Sys.sleep(0.001)
       }
+      invisible(id)
     },
     browse = browse
   )
